searchNodes=[{"ref":"RecoverableStream.html","title":"RecoverableStream","type":"module","doc":"By extracting evaluation of the source stream into a separate process RecoverableStream provides a way to isolate upstream errors and recover from them. This module contains public API."},{"ref":"RecoverableStream.html#run/2","title":"RecoverableStream.run/2","type":"function","doc":"Evaluates passed stream_fun/0 inside a new Task then runs produced stream, forwarding data back to the caller. Returns a new Stream that gathers data forwarded by the Task. Data is forwarded element by element. Batching is to be implemented explicitly. For example Postgrex.stream/3 sends data in chunks by default. Stream function stream_fun/0 must be a function that accepts one or two arguments. The first argument is either nil or the last value received from a stream before recovery. The second argument is an arbitrary term passed from wrapper_fun/0 The function should return a Stream (although, any Enumerable could work). Example iex&gt; gen_stream_f = fn ...&gt; nil -&gt; Stream.iterate(1, fn x when x &lt; 2 -&gt; x + 1 end) ...&gt; x -&gt; Stream.iterate(x + 1, &amp;(&amp;1+1)) ...&gt; end iex&gt; RecoverableStream.run(gen_stream_f) ...&gt; |&gt; Stream.take(4) ...&gt; |&gt; Enum.into([]) [1, 2, 3, 4] Options :retry_attempts (defaults to 1) the total number of times error recovery is performed before an error is propagated. Retries counter is not reset upon a successful recovery! :task_supervisor either pid or a name of Task.Supervisor to supervise a stream-reducer Task. (defaults to RecoverableStream.TaskPool) See RecoverableStream.TasksPool.child_spec/1 for details. :wrapper_fun is a funciton that wraps a stream reducer running inside a Task (defaults to fun f -&gt; f.(%{}) end). Useful when the stream_fun/0 must be run within a certain context. E.g. Postgrex.stream/3 only works inside Postgrex.transaction/3. See Readme for a more elaborate example."},{"ref":"RecoverableStream.html#t:inner_reduce_fun/0","title":"RecoverableStream.inner_reduce_fun/0","type":"type","doc":""},{"ref":"RecoverableStream.html#t:last_value_t/0","title":"RecoverableStream.last_value_t/0","type":"type","doc":""},{"ref":"RecoverableStream.html#t:run_option/0","title":"RecoverableStream.run_option/0","type":"type","doc":""},{"ref":"RecoverableStream.html#t:stream_arg_t/0","title":"RecoverableStream.stream_arg_t/0","type":"type","doc":""},{"ref":"RecoverableStream.html#t:stream_fun/0","title":"RecoverableStream.stream_fun/0","type":"type","doc":""},{"ref":"RecoverableStream.html#t:wrapper_fun/0","title":"RecoverableStream.wrapper_fun/0","type":"type","doc":""},{"ref":"RecoverableStream.TasksPool.html","title":"RecoverableStream.TasksPool","type":"module","doc":"A default Supervisor for tasks spawned by RecoverableStream."},{"ref":"RecoverableStream.TasksPool.html#child_spec/1","title":"RecoverableStream.TasksPool.child_spec/1","type":"function","doc":"A template child_spec for a custom Task.Supervisor. Example iex&gt; {:ok, _} = Supervisor.start_child( ...&gt; RecoverableStreamEx.Supervisor, ...&gt; RecoverableStream.TasksPool.child_spec(:my_sup)) ...&gt; RecoverableStream.run( ...&gt; fn x -&gt; Stream.repeatedly(fn -&gt; x end) end, ...&gt; task_supervisor: :my_sup ...&gt; ) |&gt; Stream.take(2) |&gt; Enum.into([]) [nil, nil]"},{"ref":"readme.html","title":"RecoverableStreamEx","type":"extras","doc":"RecoverableStreamEx Streams are the best way to handle &quot;potentially infinite&quot; data: imagine a query scanning a table with a filter and returning rows in batches. E.g. Postgrex.stream/4 provides an interface like that. But what happens if a connection to a database stutters? The entire stream would fail. Then an app should retry. But what if retrying from the beginning is expensive? It could mean that the database has to re-scan hundreds of GBs. Can we avoid that? Can we make it transparent for client-code? RecoverableStream moves stream evaluation into a separate process, to isolate errors. It could be set to retry up to a specified number of times, from the last retrieved value. If the stream evaluation keeps failing, an error is propagated to a caller."},{"ref":"readme.html#basic-usage","title":"RecoverableStreamEx - Basic usage","type":"extras","doc":"RecoverableStream.run function takes 2 parameters: a stream re-creation fun and a list of options. Stream creation function is called with the last element, obtained from a stream or nil and must return a new stream: gen_stream_f = fn # will fail after 10 elements nil -&gt; Stream.iterate(1, fn x when x &lt; 10 -&gt; x + 1 end) x -&gt; Stream.iterate(x + 1, &amp;(&amp;1+1)) end res = RecoverableStream.run(gen_stream_f, retry_attempts: 5) |&gt; Stream.take(20) |&gt; Enum.into([]) assert Enum.into(1..n, []) == res"},{"ref":"readme.html#a-practical-example","title":"RecoverableStreamEx - A Practical Example","type":"extras","doc":"Assuming there is a table defined like: CREATE TABLE IF NOT EXISTS recoverable_stream_test (a integer PRIMARY KEY, b integer, c integer) RecoverableStream can be used to wrap Postgrex.stream. Becuase Postgrex API expects all stream operations to be conducted within a transaction. An additional wrapper_fun parameter allows us to comply. The wrapper fun is allowed to pass additional metadata to the stream creation function. gen_stream = fn last_val, %{conn: conn} -&gt; [from, _, _] = last_val || [0, 0, 0] Postgrex.stream(conn, &quot;SELECT a, b, c FROM recoverable_stream_test WHERE a &gt; $1 ORDER BY a&quot;, [from]) |&gt; Stream.flat_map(fn(%Postgrex.Result{rows: rows}) -&gt; rows end) end wrapper_fun = fn f -&gt; Postgrex.transaction(db_pid, fn(conn) -&gt; f.(%{conn: conn}) end) end RecoverableStream.run(gen_stream, wrapper_fun: wrapper_fun) |&gt; Stream.each(&amp;IO.inspect/1) |&gt; Stream.run"}]